@@ -187,8 +187,8 @@ module NumRu
       new(val, uni)
     end
 
-    @@supported_calendars = [nil,"gregorian", "standard","noleap","365_day",
-                            "360_day"]
+    @@supported_calendars = [nil,"gregorian", "standard", "proleptic_gregorian", 
+                            "noleap", "365_day", "360_day"]
 
     def self::supported_calendar?(cal)
       @@supported_calendars.include?(cal)
@@ -201,6 +201,7 @@ module NumRu
     # * date (Date or DateTime)
     # * units (Units or String) : units of the UNumeric to be created
     def self::from_date(date, units, calendar=nil)
+      date = date.to_datetime
       sunits = units.to_s
       /(.*) *since *(.*)/ =~ sunits
       if (!$1 or !$2)
@@ -216,6 +217,9 @@ module NumRu
         case calendar
         when nil, "gregorian", "standard"
           time = Units['days'].convert( date-since, tun )
+        when "proleptic_gregorian"
+          since = since.new_start(Date::GREGORIAN) # convert to Proleptic Gregorian
+          time = Units['days'].convert( date-since, tun )
         when "noleap", "365_day"
           since_yday = since - DateTime.new(since.year,1,1) # day number of year (0..364)
           since_yday = since_yday - 1 if( since.leap? && since.mon > 2 )
@@ -224,14 +228,19 @@ module NumRu
             if date_yday >= 60.0 # after Mar1
               date_yday = date_yday - 1
             elsif date_yday >= 59.0 # Feb29
-              raise "Feb.29 is specified, but calendar is #{calendar}."
+              raise("Feb.29 is specified, but calendar is #{calendar}.")
             end
           end
           days = (date.year - since.year)*365 + (date_yday - since_yday)
           time = Units['days'].convert( days, tun )
         when "360_day" # does not work perfectly
-          days = (date.year - since.year)*360 + (date.mon - since.mon)*30 + (date.day - since.day)
-          time = Units['days'].convert( days, tun )
+          if date.day == 31
+            raise("day=31 is specified, but calendar is #{calendar}.")
+          end
+          days = (date.year-since.year)*360 + (date.mon-since.mon)*30 + 
+                 (date.day-since.day) + Rational(date.hour-since.hour,24) + 
+                 Rational(date.min-since.min,1440) + Rational(date.sec-since.sec,86400)
+          time = Units['days'].convert( days.to_f, tun )
         else
           #raise("Unrecognized calendar: #{calendar}")
           return nil
@@ -313,6 +322,11 @@ module NumRu
       elsif( tun =~ Units['days since 0001-01-01'] )
         case calendar
         when nil, "gregorian", "standard"
+          # default: Julian calendar before 1582-10-15, Gregorian calendar afterward
+          datetime = since + tun.convert( time, Units['days'] )
+        when "proleptic_gregorian"
+          # Gregorian calendar extended to the past
+          since = DateTime.parse(UNumeric::before_date_parse(sincestr),false,Date::GREGORIAN)
           datetime = since + tun.convert( time, Units['days'] )
         when "noleap", "365_day"
           since_yday = since - DateTime.new(since.year,1,1) # day number of year (0..364)
@@ -327,7 +341,12 @@ module NumRu
           days = (since.mon-1)*30 + since_day + tun.convert( time, Units['days'] )
           year = since.year + (days/360).to_i
           mon = ((days%360)/30).to_i + 1
-          datetime = DateTime.new(year,mon,1) + days%30
+          datetime = DateTime.new(year,mon,1) + days%30 # Feb29->Mar1,Feb30->Mar2
+          #datetime = DateTime.new(year,mon,days%30 + 1) # stops if Feb29,Feb30
+          if datetime.mon != mon # Feb29,Feb30
+            $stderr.print("cannot convert #{year}-#{mon}-#{(days%30+1).to_i} to DateTime instance\n")
+            return nil
+          end
         else
           #raise("Unrecognized calendar: #{calendar}")
           return nil
